@@ -339,17 +339,23 @@ class AttributeContainer(metaclass=AttributeContainerMeta):
                 raise ValueError("Attribute {} specified does not exist".format(attr_name))
             setattr(self, attr_name, attr_value)
 
-    def serialize(self, null_check=True) -> Dict[str, Dict[str, Any]]:
+    def _container_serialize(self, null_check: bool = True) -> Dict[str, Dict[str, Any]]:
         """
         Serialize attribute values for DynamoDB
         """
         attribute_values: Dict[str, Dict[str, Any]] = {}
         for name, attr in self.get_attributes().items():
             value = getattr(self, name)
-            if isinstance(value, MapAttribute) and not value.validate():
+            if isinstance(value, MapAttribute) and not value.validate(null_check=null_check):
                 raise ValueError("Attribute '{}' is not correctly typed".format(name))
 
-            attr_value = attr.serialize(value) if value is not None else None
+            if value is not None:
+                if isinstance(attr, MapAttribute):
+                    attr_value = attr.serialize(value, null_check=null_check)
+                else:
+                    attr_value = attr.serialize(value)
+            else:
+                attr_value = None
             if null_check and attr_value is None and not attr.null:
                 raise ValueError("Attribute '{}' cannot be None".format(name))
 
@@ -357,7 +363,7 @@ class AttributeContainer(metaclass=AttributeContainerMeta):
                 attribute_values[attr.attr_name] = {attr.attr_type: attr_value}
         return attribute_values
 
-    def deserialize(self, attribute_values: Dict[str, Dict[str, Any]]) -> None:
+    def _container_deserialize(self, attribute_values: Dict[str, Dict[str, Any]]) -> None:
         """
         Sets attributes sent back from DynamoDB on this object
         """
@@ -387,17 +393,9 @@ class AttributeContainer(metaclass=AttributeContainerMeta):
             raise ValueError("Cannot instantiate a {} from the returned class: {}".format(
                 cls.__name__, stored_cls.__name__))
         instance = (stored_cls or cls)(_user_instantiated=False)
-        AttributeContainer.deserialize(instance, attribute_values)
+        AttributeContainer._container_deserialize(instance, attribute_values)
         return instance
 
-    def __eq__(self, other: Any) -> bool:
-        # This is required so that MapAttribute can call this method.
-        return self is other
-
-    def __ne__(self, other: Any) -> bool:
-        # This is required so that MapAttribute can call this method.
-        return self is not other
-
 
 class DiscriminatorAttribute(Attribute[type]):
     attr_type = STRING
@@ -835,14 +833,14 @@ class MapAttribute(Attribute[Mapping[_KT, _VT]], AttributeContainer):
             if isinstance(local_attr, MapAttribute):
                 local_attr._update_attribute_paths(path_segment)
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> 'Comparison':  # type: ignore[override]
         if self._is_attribute_container():
-            return AttributeContainer.__eq__(self, other)
+            return self is other  # type: ignore
         return Attribute.__eq__(self, other)
 
-    def __ne__(self, other):
+    def __ne__(self, other: Any) -> 'Comparison':  # type: ignore[override]
         if self._is_attribute_container():
-            return AttributeContainer.__ne__(self, other)
+            return self is not other  # type: ignore
         return Attribute.__ne__(self, other)
 
     def __iter__(self):
@@ -914,8 +912,8 @@ class MapAttribute(Attribute[Mapping[_KT, _VT]], AttributeContainer):
         else:
             super()._set_attributes(**attrs)
 
-    def is_correctly_typed(self, key, attr):
-        can_be_null = attr.null
+    def is_correctly_typed(self, key, attr, *, null_check: bool = True):
+        can_be_null = attr.null or not null_check
         value = getattr(self, key)
         if can_be_null and value is None:
             return True
@@ -923,10 +921,11 @@ class MapAttribute(Attribute[Mapping[_KT, _VT]], AttributeContainer):
             raise ValueError("Attribute '{}' cannot be None".format(key))
         return True  # TODO: check that the actual type of `value` meets requirements of `attr`
 
-    def validate(self):
-        return all(self.is_correctly_typed(k, v) for k, v in self.get_attributes().items())
+    def validate(self, *, null_check: bool = False):
+        return all(self.is_correctly_typed(k, v, null_check=null_check)
+                   for k, v in self.get_attributes().items())
 
-    def serialize(self, values):
+    def serialize(self, values, *, null_check: bool = True):
         if not self.is_raw():
             # This is a subclassed MapAttribute that acts as an AttributeContainer.
             # Serialize the values based on the attributes in the class.
@@ -940,7 +939,7 @@ class MapAttribute(Attribute[Mapping[_KT, _VT]], AttributeContainer):
                         setattr(instance, name, values[name])
                 values = instance
 
-            return AttributeContainer.serialize(values)
+            return AttributeContainer._container_serialize(values, null_check=null_check)
 
         # Continue to serialize NULL values in "raw" map attributes for backwards compatibility.
         # This special case behavior for "raw" attributes should be removed in the future.
